// ----------------------------------------------------------------------------
// Copyright (C) 2018
//              David Freese, W1HKJ
//
// This file is part of pskmeter.
//
// pskmeter is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// pskmeter is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------

#ifndef CPSKDATA
#define CPSKDATA

#include "serial.h"

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#define TWOPI (2.0 * M_PI)
#define PIOVR2 (M_PI / 2.0)
#define powerconst (100.0 * sqrtf(2.0))

#define A2DSIZE 32768 //1024 // 256 //32768
#define A2D 5.0
#define NUMPTS 64

enum PSKMODE {PSK31, PSK63, IDLETEST, TEXTTEST};

class Cpskdata {
friend class cScopeTimePlot;
friend class cScopeFFTPlot;
	double raw_vals[NUMPTS];
	double normalized_vals[NUMPTS];
	double minval;
	double min;
	double max;
	PSKMODE mode;
	double fVals[NUMPTS + 1];
	double sinTable[NUMPTS + 1];
	double cosTable[NUMPTS + 1];
	double imd;
	double rms;
	double mean;
	double peak;
	double calpeak;
	double alpha;
	double targetPower;
	double targetVoltage;
	double minVoltage;
	double maxVoltage;
	double R1qro;
	double R1qrp;
	double ratio;
	double powerconv;
	double overdrive1;
	double overdrive2;
	double q[7];
	double qsum;
	double signalPower;
	double AtoD;
	bool calmode;
	bool  empty;
	bool  showrawvals;
	bool  qrp;
	int nrows;
	int ncols;
	int sizeText;
	char *szTEXT;
	char *szRMS;
	char *szIMD;
	char *szPEAK;

	bool normalize ();
	void filter ();
	void stats ();
	void dft ();

public:
	Cpskdata ();
	~Cpskdata (){};

	void data(double * rawdata);

	void Alpha(double a) {alpha = a;};
	double Alpha()  {return alpha;};

	void valR1qro (double R) {
		R1qro = R;
	};
	double valR1qro () {return R1qro;};

	void valR1qrp (double R) {
		R1qrp = R;
	};
	double valR1qrp () {return R1qrp;};

	void Qrp(bool b) {
		qrp = b;
		double R = R1qro;
		if (b)
		R = (R1qro * R1qrp)/(R1qro + R1qrp);
		ratio = 1 + R / 2.2;
		targetVoltage = sqrtf(targetPower*powerconv)/(ratio);
	}
	bool Qrp() { return qrp; }

	void Minval (double m) { minval = m / A2DSIZE;};
	double Minval () {return minval * A2DSIZE;};

	void Overdrive1(double r) {overdrive1 = pow(10.0, r/10.0);};
	double  Overdrive1() {return int(10*log10(overdrive1));};

	void Overdrive2(double r) {overdrive2 = pow(10.0, r/10.0);};
	double  Overdrive2() {return (10*log10(overdrive2));};

	double drive(double p) {
		return sqrtf(p*powerconv)/(ratio*5.0);
	}

	void targetPwr(double p) {
		targetPower = p;
		targetVoltage = sqrtf(p*powerconv)/(ratio); // see pskdata.cpp
	};
	double targetPwr() {return targetPower;};
	double targetV() {return targetVoltage;};
	double peakV() { return max * AtoD;};

	void    Mode(PSKMODE m) {mode = m;};
	PSKMODE Mode() {return mode;};

	bool isXmtng() { return !empty;};
	double Peak() { return peak;};
	double Ratio() {return ratio;};

	double error();

	int IMDstat();
	const char *szData();
	const char *szRawData();
	const char *szFilteredData();
	const char *szImd();
	const char *szRms();
	const char *szPeak();
	double SignalPwr();
	const char *szPower ();
	const char *szSpectralLines();
};

#endif

