// ----------------------------------------------------------------------------
// Copyright (C) 2018
//              David Freese, W1HKJ
//
// This file is part of pskmeter.
//
// pskmeter is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// pskmeter is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
#include <iostream>

#include "pskdata.h"

Cpskdata::Cpskdata() {
	mode = PSK31;
	ncols = 4;
	nrows = NUMPTS / ncols;
	sizeText = nrows * (5 * ncols + 1);
	szTEXT = new char[sizeText + 1];
	szRMS = new char[40];
	szIMD = new char[40];
	szPEAK = new char[40];
	for (int i = 0; i <= NUMPTS; i++) {
		raw_vals[i] = 0;
	normalized_vals[i] = 0;
		fVals[i] = 0.0;
	}
	imd = rms = calpeak = peak = 0.0;
	empty = true;
	for (int i = 0; i < NUMPTS; i++) {
		sinTable[i] = sin( TWOPI * i / NUMPTS );
		cosTable[i] = cos( TWOPI * i / NUMPTS );
	}
// default configuration values
	showrawvals = true;

	alpha = 0.5;    // simple low pass filter applied to raw data
	R1qro = 39.0;
	R1qrp = 6.8;
	qrp = false;
	ratio = 1 + R1qro / 2.2;

	AtoD = A2D;

//	calmode = false;

	powerconv = AtoD * AtoD / (2 * 50 * sqrt(2));

// These starting values are valid for QRO
// at idle
// Prms = ((Vp*ratio)^2)/(2*sqrt(2)*R) (rf power ..... not audio power)
// where Vp = measured peak audio voltage
//       ratio = (1 + R1/R2);
//       R = real part of antenna impedance, nominally 50 ohms

//	targetPower = 15.0;
//	targetVoltage = sqrtf(targetPower*powerconv)/ratio;

	overdrive1 = 0.002;
	overdrive2 = 0.01;
}

bool Cpskdata::normalize () {
	int zeroloc = 0, top;
	double step;
	double r[NUMPTS];
	for (int i = 0; i < NUMPTS; r[i++] = 0);
	min = 1.0;
	max = 0;

// find the min and max values and sample location of min value  
	for (int i = 0; i < NUMPTS; i++) {
		if (raw_vals[i] > max) 
			max = raw_vals[i];
		if (raw_vals[i] < min) {
			min = raw_vals[i];
			zeroloc = i;
		}
	}

// rotate data array to put min value at position 0
	for (int i = zeroloc,  j = 0; i < NUMPTS; i++, j++)
		r[j] = raw_vals[i];
	for (int i = 0, j = NUMPTS - zeroloc; i < zeroloc; i++, j++)
		r[j] = raw_vals[i];

// if (max < threshold) then NOT TRANSMITTING so reset all computations
	if (max <= minval) {
		for (int i = 0; i < NUMPTS; normalized_vals[i++] = 0);
		empty = true;
		rms = 0.0;
		peak = 0.0;
//    calpeak = 0.0; 
		imd = 0.0;
		for (int i = 0; i < 7; i++) q[i] = 0.0;
		qsum = 0.0;
		signalPower = 0.0;
		return false;
	}

// normalize the data array to the unit vector  
	for (int i = 0; i < NUMPTS; i++) {
		normalized_vals[i] = r[i] / max;
	}

// if (max - min) is less than threshold then data array does not contain
// a null point ... sample occurred in middle of series of 1 bits
	if ((max-min) < minval) {
		return false;
	}

// if (min > minval) then a true null was not sampled
// sample probably occurred at the leading or trailing edge of a series of 1 bits
	if (min > minval) {
		return false;
	}

// check first 1/2 cycle values for discontinuity due to data rotation & sampling
	if (mode == PSK31) {
//		top = 16; 
		top = NUMPTS / 4;
		step = 0.1;//50;
	} else {
//		top = 8;
		top = NUMPTS / 8;
		step = 0.2;//100;
	}
//	for (int i = 1; i < top; i++) {
//		if (normalized_vals[i] - normalized_vals[i-1] > step) return false;
//	}

// data rotated, normalized and checked for various sampling problems
// passed all tests, so can now be used for computing IMD, RMS, etc.
	return true;
}

// low pass filter raw data and create pseudo signal tables
// perform dft and statistical analysis
void Cpskdata::filter () {
	if (empty == true)
		peak = max;
	else
		peak = alpha * peak + (1-alpha) * max; 
	calpeak = alpha*calpeak + (1-alpha)*peak;
	if (mode == PSK31) {
		if (empty == true) {
			for (int i = 0; i <= NUMPTS / 4; i++)
				fVals[i] = normalized_vals[i];
			empty = false;
		} else {
			for (int i = 0; i <= NUMPTS / 4; i++)
				fVals[i] = alpha * fVals[i] + (1.0 - alpha) * normalized_vals[i];
		}
		for (int i = 0; i <= NUMPTS / 4; i++)
			fVals[NUMPTS/2 + i] = -fVals[i];
		for (int i = 0; i < NUMPTS / 4; i++) {
			fVals[NUMPTS/2 - i] = fVals[i];
			fVals[NUMPTS-i] = -fVals[i];
		}
	} else {
		if (empty == true) {
			for (int i = 0; i <= NUMPTS / 8; i++)
				fVals[i] = normalized_vals[i];
			empty = false;
		} else {
			for (int i = 0; i <= NUMPTS / 8; i++)
				fVals[i] = alpha * fVals[i] + (1.0 - alpha) * normalized_vals[i];
		}
		for (int i = 0; i <= NUMPTS / 8; i++) {
			fVals[NUMPTS/2 + i] = fVals[i];
			fVals[NUMPTS/4 + i] = fVals[NUMPTS / 2 + NUMPTS / 4 + i] = -fVals[i];
		}
		for (int i = 0; i < NUMPTS / 8; i++) {
			fVals[NUMPTS / 4 - 1] = fVals[NUMPTS / 2 + NUMPTS / 4 - i] = fVals[i];
			fVals[NUMPTS / 2 - i] = fVals[NUMPTS - i] = fVals[i];
		}
	}
}

// dft - discrete fourier transform analysis
void Cpskdata::dft()
{
	int si; // sample increment
	double a = 0;
	double b = 0;
	double mult = (2.0 / NUMPTS);


	for (int i = 0; i < 7; i++)
		q[i] = 0.0;

	qsum = 0.0;
	signalPower = 0.0;
	for (int j = 0; j < 7; j++) {// += 2) { // fundamental, 3rd, 5th & 7th harmonics
		a = b = 0.0;
		for (int i = 0; i <= NUMPTS; i++) {
			if (mode == PSK31)
				si = ((j+1)*i) % NUMPTS;
			else
				si = (2*(j+1)*i) % NUMPTS;
			a += fVals[i]*sinTable[si];
			b += fVals[i]*cosTable[si];
		}
		a *= mult;
		b *= mult;
		q[j] = (a*a + b*b);  // power in jth harmonic
	}

	double qnom = q[0];
	q[0] = 1.0;
	for (int i = 1; i < 7; i++) {// += 2) {
			q[i] /= qnom;
			qsum += q[i]; // sum of all harmonic powers
		}

	signalPower = qnom * (1.0 + qsum)*(ratio * ratio * powerconv);
}

//statistical analysis
void Cpskdata::stats() {  
	static  double err = 0.0;
	rms = 0.0;
	int nsamples = (mode == PSK31 ? (NUMPTS / 4) : (NUMPTS/8));
// compute value of rms based on first 16/8 samples of the normalized waveform
	for (int i = 0; i < nsamples; i++) {
		err = fVals[i] - sinTable[i];
		rms += err * err;
//    mean += err;
	}
//  mean = mean / nsamples;
	rms = sqrtf(rms)/(nsamples - 1);
}

void Cpskdata::data(double *rawdata) {
	for (int i = 0; i < NUMPTS; i++)
		raw_vals[i] = rawdata[i];

	if (normalize() == false)
		return; // data set does not contain valid 1/4 sin wave
	filter ();
	dft ();
	stats ();
}

const char *Cpskdata::szRawData() {
	char szt[50];
	static string szTEXT;
	szTEXT.clear();
	for (int n = 0; n < NUMPTS/2; n += 4) {
		sprintf(szt, "%6d %6d %6d %6d\n",
			(int)(A2DSIZE * fVals[ n ]), 
			(int)(A2DSIZE * fVals[n+1]), 
			(int)(A2DSIZE * fVals[n+2]), 
			(int)(A2DSIZE * fVals[n+3]));
		szTEXT.append(szt);
	}
	return szTEXT.c_str();
}

const char *Cpskdata::szFilteredData() {
	char *szt;
	double antv = peak * ratio;
	static string szTEXT;
	szTEXT.clear();
	for (int n = 0; n < NUMPTS / 2; n += 8) {
		sprintf( szt, "%6.1f %6.1f %6.1f %6.1f %6.1f %6.1f %6.1f %6.1f\n",
			fVals[ n ]*antv, fVals[n+1]*antv, fVals[n+2]*antv, fVals[n+3]*antv,
			fVals[n+4]*antv, fVals[n+5]*antv, fVals[n+6]*antv, fVals[n+7]*antv);
		szTEXT.append(szt);
	}
	return szTEXT.c_str();
}

const char * Cpskdata::szRms() {
	sprintf(szRMS, "%4.2f", 100.0*rms);
	return szRMS;
}

const char * Cpskdata::szPeak() {
	sprintf(szPEAK, "%4.2f", peak);
	return szPEAK;
}

const char *Cpskdata::szImd() {
	if (q[0] == 0.0) 
		sprintf(szTEXT, " ------");
	else
		sprintf(szTEXT, "%4.1f", 10.0*log10(qsum)); // power ratio of harmonic/fundamental
	return szTEXT;
}

int Cpskdata::IMDstat() {
	if (q[0] == 0.0) return 0;
	if (qsum > overdrive2) 
		return 2;
	if (qsum > overdrive1) 
		return 1;
	return 0;
}

double Cpskdata::SignalPwr() {
	if (empty) return 0.0;
	return signalPower;
}

const char *Cpskdata::szPower() {
	sprintf(szTEXT,"%4.1f", signalPower);
	return szTEXT;
}

const char *Cpskdata::szSpectralLines() {
	sprintf(szTEXT,"%7.5f,%7.5f,%7.5f,%7.5f\n%7.5f,%7.5f,%7.5f",
	q[0], q[1], q[2], q[3], q[4], q[5], q[6]);
	return szTEXT;
}
