// ----------------------------------------------------------------------------
// Copyright (C) 2018
//              David Freese, W1HKJ
//
// This file is part of pskmeter.
//
// pskmeter is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// pskmeter is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------

#include "support.h"

bool debugging = false;

extern void readConfigFile();
extern void cbSampleTimer(void *);
extern void readdata();

Fl_Help_Dialog *help = NULL;
Fl_Window *about = NULL;
Fl_Double_Window *wConfig = 0;
Fl_Double_Window *wSimulation = 0;

Cserial   *pskserial = 0;
Cpskdata  *pskdata;

bool bViewScope = true;
bool bQRP = false;
bool bAutoDrive = false;
bool bTestmode = false;
bool bEnabled = true;

double sampletime = 0.5;
double saveVol;
double err;

int  iNbrCommPorts  = 0;
int  iCommPort = 0;
int testmode = 0;  // off = 0; idle signal = 1; text signal = 2;

char szCommPort[80];
char szVersion[80];
char progdir[160];


bool testMeter() {
	char buff[1024];
	int n;
	if (pskserial->WriteByte('v') == false)
		return false;
	memset (buff, 0, 1024);
	n = pskserial->ReadBuffer(buff, 1024);
	if (!n)
		return false;
	buff[1023] = 0;
	if (strstr(buff,"PSKMETER") != 0)
		return true;
	return false;
}

bool initCommPort() {
	pskserial->Device(szCommPort);
	pskserial->Baud(19200);
	pskserial->Stopbits(1);
	pskserial->RTSCTS(false);
	if (pskserial->OpenPort()) {
		if (testMeter())
			return true;
			else {
			pskserial->ClosePort();
			fl_message("Psk Meter not on specified port!");
			return false;
		}
	}
}

void initProgram(void *d) {
	char *executable = (char *)d; 
	strcpy (progdir, executable);
	char *p = strrchr(progdir,'/');
	if (!p) p= strrchr(progdir, '\\');
	if (p) *(p+1) = 0;
	
	if (wSimulation == 0) wSimulation = make_SimulateDialog();

	pskserial = new Cserial;
	pskdata = new Cpskdata;
	wdgSignalPlot->setData(pskdata);
	wdgSpectrumPlot->setData(pskdata);

	readConfigFile();

	initCommPort();

	readdata();
}

#ifdef __WIN32__
#define exp10(v) exp((v)*log(10))
#endif

void testdata() {
	double vals[NUMPTS];
	if (wSimulation->shown() == false) {
		wSimulation->position(
			pskscope_dialog->x(), pskscope_dialog->y() + pskscope_dialog->decorated_h());
		wSimulation->show();
	}

	double p = exp10(sldrSimulate->value()/20.0);
	double vp = 5.0;
	double sig = 0;
	int shift = (int)(64.0 * random() / RAND_MAX);

	for (int i = 0; i < NUMPTS; i++) {
		sig = sin(TWOPI*(i + shift)/NUMPTS);
		for (int q = 3; q <= 7; q += 2)
			sig += (9*p/(q*q))*sin(TWOPI*q*(i + shift)/NUMPTS);
		sig = fabs((int)(sig * A2DSIZE));
		vals[i] = vp * sig / A2DSIZE;
	}
	pskdata->data(vals);
}

void readdata() {
	double vals[NUMPTS];
	int n = 0;
	if (wSimulation->shown() == true)
		wSimulation->hide();
	if (iCommPort == 0 || bEnabled == false) {
		for (n = 0; n < NUMPTS; n++) vals[n] = 0;
	} else {
		pskserial->WriteByte('s');
//		n = pskserial->ReadBuffer (vals, NUMPTS * sizeof(int));
	}
	pskdata->data(vals); 
}

void cbSampleTimer(void* d) {
	if (testmode) {
		if (bTestmode == 0) {
			bTestmode = 1;
		}
		testdata();
	} else {
		if (bTestmode) {
			bTestmode = 0;
		}
		readdata();
	}
	char temp[40];
	sprintf(temp,"db %s", pskdata->szImd());
	txtIMD->value(temp);
	sprintf(temp,"%%  %s", pskdata->szRms());
	txtRMS->value (temp);
	sprintf(temp,"Vp %s", pskdata->szPeak());
	txtPeak->value (temp);
	sprintf(temp,"Pw %s",pskdata->szPower());
	txtPower->value (temp);

	wdgSignalPlot->redraw();
	wdgSpectrumPlot->redraw();
	txtRawData->value (pskdata->szRawData());
	txtFilteredData->value (pskdata->szFilteredData());

	Fl::repeat_timeout(sampletime, cbSampleTimer);
}

void clean_exit() {
	if (pskserial)
		pskserial->ClosePort();
	exit(0);
}

void cb_mnuExit(Fl_Menu_* m, void* d) {
	clean_exit();
}

void cb_mnuConfig(Fl_Menu_* m, void* d) {
}

void cb_mnuContext(Fl_Menu_* m, void* d) {
}

void cb_mnu05sec(Fl_Menu_* m, void* d){
	sampletime = 0.5;
}

void cb_mnu10sec(Fl_Menu_* m, void* d){
	sampletime = 1.0;
}

void cb_mnu20sec(Fl_Menu_* m, void* d){  
	sampletime = 2.0;
}

void cb_Mode(PSKMODE d) {
	if (d > PSK63) {
		testmode = d - PSK63;
		pskdata->Mode(PSK31);
	} else {
		testmode = 0;
		pskdata->Mode(d);
	}
}

void cb_mnuConfigGeneral(Fl_Menu_* m, void* d){
	if (!wConfig) wConfig = make_ConfigDialog();
	valR1qro->value(pskdata->valR1qro());
	valR1qrp->value(pskdata->valR1qrp());
	valSignalMin->value(pskdata->Minval());
	valRed->value(pskdata->Overdrive2());
	valYellow->value(pskdata->Overdrive1());
	valTargetPwr->value(pskdata->targetPwr());
	valFilter->value(pskdata->Alpha());
	inpCommPort->value(szCommPort);
	btn_qrp_jumpered->value(pskdata->Qrp());
	wConfig->show();
}

void setParameters() {
	strcpy(szCommPort,inpCommPort->value());
	initCommPort();
	pskdata->valR1qro(valR1qro->value());
	pskdata->valR1qrp(valR1qrp->value());
	pskdata->Minval(valSignalMin->value());
	pskdata->Overdrive2(valRed->value());
	pskdata->Overdrive1(valYellow->value());
	pskdata->Alpha(valFilter->value());
	pskdata->targetPwr(valTargetPwr->value());
	pskdata->Qrp(btn_qrp_jumpered->value());
}

static char fileid[] = PACKAGE_TARNAME;

void writeConfigFile() {
	char fname[160];
	strcpy (fname, progdir);
	strcat (fname, "pskscope.ini");
	FILE *fConfig = fopen (fname, "w");
	if (!fConfig) return;
	if (strlen(szCommPort) == 0) strcpy(szCommPort, "NONE");
	fprintf(fConfig,"%s-%s\n", fileid, PACKAGE_VERSION);
	fprintf(fConfig,"%s\n", szCommPort);
	fprintf(fConfig, "%f\n", pskdata->Minval());
	fprintf(fConfig, "%f\n", pskdata->Overdrive2());
	fprintf(fConfig, "%f\n", pskdata->Overdrive1());
	fprintf(fConfig, "%f\n", pskdata->valR1qro());
	fprintf(fConfig, "%f\n", pskdata->valR1qrp());
	fprintf(fConfig, "%f\n", pskdata->Alpha());
	fprintf(fConfig, "%f\n", pskdata->targetPwr());
	fprintf(fConfig, "%d\n", pskdata->Qrp());
	fclose(fConfig);
}

void readConfigFile() {
	char fname[160];
	strcpy (fname, progdir);
	strcat (fname, "pskscope.ini");
	FILE *fConfig = fopen (fname, "r");
	char fid[80];
	char ver[80];
	float fval;
	int i;

	if (!fConfig) return;
	sprintf(ver,"%s-%s", fileid, PACKAGE_VERSION);
	if (!fscanf(fConfig, "%s\n", fid)) return;
	if (!strcmp(fid,ver) == 0) {
		fl_message("pskscope.ini wrong version\nPlease save new configuration file");
		return;
	} else {
		if (!fscanf(fConfig,"%s\n", szCommPort)) {
			strcpy(szCommPort, "NONE");
//			return;
		}
		if (inpCommPort) inpCommPort->value(szCommPort);

		if (!fscanf(fConfig,"%f\n", &fval)) return;
		pskdata->Minval(fval);
		if (valSignalMin) valSignalMin->value(fval);

		if (!fscanf(fConfig,"%f\n", &fval)) return;
		pskdata->Overdrive2(fval);
		if (valRed) valRed->value(fval);

		if (!fscanf(fConfig,"%f\n", &fval)) return;
		pskdata->Overdrive1(fval);
		if (valYellow) valYellow->value(fval);

		if (!fscanf(fConfig,"%f\n", &fval)) return;
		pskdata->valR1qro(fval);
		if (valR1qro) valR1qro->value(fval);

		if (!fscanf(fConfig, "%f\n", &fval)) return;
		pskdata->valR1qrp(fval);
		if (valR1qrp) valR1qrp->value(fval);

		if (!fscanf(fConfig,"%f\n", &fval)) return;
		pskdata->Alpha(fval);
		if (valFilter) valFilter->value(fval);

		if (!fscanf(fConfig,"%f\n", &fval)) return;
		pskdata->targetPwr(fval);
		if (valTargetPwr) valTargetPwr->value(fval);

		if (!fscanf(fConfig, "%d\n", &i)) return;
		pskdata->Qrp(i);
		if (btn_qrp_jumpered) btn_qrp_jumpered->value(i);

	}
	fclose(fConfig);
}

void cb_btnConfigOK(Fl_Button *b, void * d) {
	setParameters();
	wConfig->hide();
}

void cb_btnConfigSave(Fl_Button *b, void * d) {
	setParameters();
	writeConfigFile();
	wConfig->hide();
}

void cb_btnConfigCancel(Fl_Button *b, void *d) {
	wConfig->hide();
}

void cb_wMain(Fl_Double_Window *w, void *d) {
	if (Fl::event_key() == FL_Escape)
		return;
	clean_exit();
}

//  ============================ About dialog =========================

void cb_mnuAbout(Fl_Menu_ *mnu, void *d)
{
	if (!about) {
		sprintf (szVersion, "\
Psk Scope %s\n\
Use with PskMeter\n\
Free Hamware by W1HKJ\n\n\
Report problems to:", PACKAGE_VERSION);
		about = newAboutDialog ();
		lblVersion->label (szVersion);
	}
	about->show ();
}

void cb_mnuHelpContext (Fl_Menu_ *mnu, void *d)
{
#ifdef __WIN32__
	PROCESS_INFORMATION processInfo;
	STARTUPINFO startupInfo;
	::ZeroMemory(&startupInfo, sizeof(startupInfo));
	startupInfo.cb = sizeof(startupInfo);
	char szCommandLine[160] = "\\Program Files\\Internet Explorer\\IEXPLORE.EXE ";
	strcat(szCommandLine, progdir);
	strcat(szCommandLine, "help\\index.html");
	
	if (!::CreateProcess(
			NULL, szCommandLine,
			NULL, NULL, 
			false, 0, 
			NULL, NULL, 
			&startupInfo, &processInfo))
		fl_message ("Cannot find Internet Explorer");
#else
	fl_message("For help:\n\n"
						 "See  pskscope/help/index.html\n\n");
#endif
	return;
}

void cb_Active(Fl_Check_Button *btn, void *d)
{
	bEnabled = btn->value();
}
